/*
   Version information
    v1.3 included Web server code, clock display, character printout, updated control routine
    v1.4 NTP reset on web page, print routine for text strings
    v1.5 implement vertical scroll with clockwise turned characters
    v1.6 partial scroll up, down (along lon axis), background array for underlay
    v1.7 AutoBrightness via light sensor on A0
    v1.8 scroll function for background
    v1.9 experimental: brightness, changes in all functions . reLight function

    v1.95 Bug causes reset - test version - check reLight funtion
      initialize A0 for Input in Setup
    v1.96 repaired reLight function crashed due to brightness variable rogue (now devided by 104 to make sure var is < 10
      new web server functionality
    v1.97 web page not yet working (enter scrolltext), implement serial print of screen
    v1.98 Text entry works now - still too basic, web page to be improved
    v1.99 Rebuild array buffer - make this a dimension in variable, i.e. layer number, implemented '.' and '*'
    v2.0 function to set colors with single function call, candle effect implemented
    v2.01 Implemented OTA functions
    v2.02 web page direct choice for effect, larger font for web page
    todo:
    Nice web page
    Web page with direct choice of effect
    Tetris
    IR remote control (also for trétris :)
    enable stepwise task performance (e.g. progress tasks in each loop) - e.g. make progress round call and step in case it is indicated
    functions for multitasking - i.e. stepwise - classses?
    sound?


*/

/*
   Includes
*/

#define SOURCECODEVERSION "V2.02"

#include <ESP8266WiFi.h>

#include <ESP8266WiFiMulti.h>
#include <ESP8266mDNS.h>
#include <ESP8266WebServer.h>
#include <TimeLib.h>
#include <Adafruit_NeoPixel.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

/*
   Definitions

*/

//#define DEBUG


#define MAXCHOICE 10
#define LEDCOUNT 294
#define STANDARDDELAY 20
#define CHARWIDTH 9        // x width of the characters
#define ASCIIOFFSET 87      // delta to enable to use ascii codes and translate into index for displayCharacters[] field
#define ASCIIZEROCHAR 48    // delta to enable to use ascii characters ("7", "8", "9") for serial connection control of clock  
//aligned to the digital ports where the buttons are connected to

#define TEXTPOSITION -1   // vertical position of topmost pixels of character.
#define CHARHEIGHT  7 // y size of characterset
#define CHARNUMBER 41

// scroll definitions
#define SCROLLUP 0
#define SCROLLDOWN 1
#define SCROLLLEFT 2
#define SCROLLRIGHT 3
#define EMPTYSCROLL 0
#define BUFFERSCROLL 1
#define LOOPSCROLL 2
#define STANDARDSCROLLSPEED 5
#define CHAR_1_POSITION 1 // x position of character 1
#define CHAR_2_POSITION 9 // x position of character 2
#define CHAR_3_POSITION 17 // x position of character 3
#define CHAR_4_POSITION 24 // x position of character 4
#define CHAR_5_POSITION 33 // x position of character 5
#define FASTSCROLL 0 // delay for scroll loop
#define SNOWLOOP 10000 // delay timing for snowfallscroll

#define FLAMEVARIATION 60
#define FLAMEBASE 170
#define FLAMEREDNESS 90
#define OUTERFLAMEPIXELS 17  //number of pixels in the outer candleflame
#define INNERFLAMEPIXELS 12  //number of pixels in the inner candleflame
#define INNERINTENSITY 20

// Screen definitions
#define SCREENX 42
#define SCREENY 7

#define MAXLAYERS 3
#define LAYER1 0
#define LAYER2 1
#define LAYER3 2
#define STANDARDTEXTLAYER 0
#define STANDARDFXLAYER 2

#define YMAX 6
#define XMAX 41

#define PIN D2

#define COLON 37
#define BLANK 36

#define BACKGROUND false
#define FOREGROUND true

#define MAXBRIGHTNESS 10
#define BRIGHTNESSIMPACT 10
#define GREENTONE 0,250,150
#define LIGHTTONE 200,200,250
#define BLUETONE 0,50,255
#define LEDCLOCKGRAVITY -0.8
#define FRICTION 0.99
#define GRAVITYON true
/*
   Funtion prototypes
*/

void handleRoot();              // function prototypes for HTTP handlers

void handleLED();

void handleNotFound();

void drawScreen(boolean showscreen);

void hideScreen();

void lightOnColor(byte redtone, byte greentone, byte bluetone);

void digitalClockDisplay();

void sendNTPpacket(IPAddress &address);

void bounce();

void drawScreen(boolean);

void scrollScreen(byte scrolldirection, byte scrolltype );

void pixelPrintChar(byte charascii, byte redtone, byte greentone, byte bluetone, byte scrolldirection, byte scrollspeed , boolean drawscreen);

void pixelPrintChar(byte whichlayer, byte charascii, byte redtone, byte greentone, byte bluetone, byte scrolldirection, byte scrollspeed , boolean drawscreen);

/*
   Object instantiation
*/

ESP8266WiFiMulti wifiMulti;     // Create an instance of the ESP8266WiFiMulti class, called 'wifiMulti'

ESP8266WebServer server(80);    // Create a webserver object that listens for HTTP request on port 80

WiFiUDP Udp;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(LEDCOUNT, PIN, NEO_GRB + NEO_KHZ800);


/*
   global Variables
*/


time_t prevDisplay = 0; // when the digital clock was displayed

time_t getNtpTime();

static const char ntpServerName[] = "us.pool.ntp.org";
const int timeZone = 1;     // Central European Time

unsigned int localPort = 8888;  // local port to listen for UDP packets


int whichChoice = 5;
int previousChoice = 0;


float xPos = 1; float yPos = 1;
float xVel = 1; float yVel = 0.5;


int brightness = 1;

byte lastsecond = second();
byte lastminute = minute() % 10;
byte lastminuteten = minute() / 10;
byte lasthour = hour() % 10;
byte lasthourten = hour() / 10;

int snowloop = SNOWLOOP;
int snowheight = 0;

/*
   Array that holds screen, colors, each pixel is represented by a triple for RGB
*/


const byte pixelCandle[LEDCOUNT][3] = {
{ 1, 0, 0 },       { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 0, 0, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 1, 1 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 1, 1, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }, 
{ 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 1, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }
};

const byte candleOuterFlame[OUTERFLAMEPIXELS][2]={
  {33,2},{34,2},{35,2},{36,1},{37,1},{38,1},{39,2},{40,2},{41,3},{40,3},{39,3},{38,4},{37,4},{36,5},{35,5},{34,5},{33,4}
};

const byte candleInnerFlame[INNERFLAMEPIXELS][2]={
  {33,3},{34,3},{34,4},{35,3},{35,4},{36,2},{36,3},{36,4},{37,3},{37,2},{38,2},{38,3}
};


byte pixelArrayFinal[LEDCOUNT][3] = {
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
  { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }

};


byte pixelArray[MAXLAYERS][LEDCOUNT][3] = {{
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }

  },

  {
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }

  },

  {
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }

  }
};


byte pixelArrayBuffer[MAXLAYERS][SCREENX][3] = {{
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }
  },
  {
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }
  },
  {
    { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 },      { 0, 0, 0 }
  }
};

const int pixel2strip [LEDCOUNT] = {
  293,   292,  291,  290,  289,  288,  287,  286,  285,  284,  283,  282,  281,  280,  279,  278,  277,  276,  275,  274,  273,  272,  271,  270,  269,  268,  267,  266,  265,  264,  263,  262,  261,  260,  259,  258,  257,  256,  255,  254,  253,  252,
  210,  211,  212,  213,  214,  215,  216,  217,  218,  219,  220,  221,  222,  223,  224,  225,  226,  227,  228,  229,  230,  231,  232,  233,  234,  235,  236,  237,  238,  239,  240,  241,  242,  243,  244,  245,  246,  247,  248,  249,  250,  251,
  209,  208,  207,  206,  205,  204,  203,  202,  201,  200,  199,  198,  197,  196,  195,  194,  193,  192,  191,  190,  189,  188,  187,  186,  185,  184,  183,  182,  181,  180,  179,  178,  177,  176,  175,  174,  173,  172,  171,  170,  169,  168,
  126,  127,  128,  129,  130,  131,  132,  133,  134,  135,  136,  137,  138,  139,  140,  141,  142,  143,  144,  145,  146,  147,  148,  149,  150,  151,  152,  153,  154,  155,  156,  157,  158,  159,  160,  161,  162,  163,  164,  165,  166,  167,
  125,  124,  123,  122,  121,  120,  119,  118,  117,  116,  115,  114,  113,  112,  111,  110,  109,  108,  107,  106,  105,  104,  103,  102,  101,  100,  99,   98,   97,   96,   95,   94,   93,   92,   91,   90,   89,   88,   87,   86,   85,   84,
  42,   43,   44,   45,   46,   47,   48,   49,   50,   51,   52,   53,   54,   55,   56,   57,   58,   59,   60,   61,   62,   63,   64,   65,   66,   67,   68,   69,   70,   71,   72,   73,   74,   75,   76,   77,   78,   79,   80,   81,   82,   83,
  41,   40,   39,   38,   37,   36,   35,   34,   33,   32,   31,   30,   29,   28,   27,   26,   25,   24,   23,   22,   21,   20,   19,   18,   17,   16,   15,   14,   13,   12,   11,   10,   9,  8,  7,  6,  5,  4,  3,  2,  1,  0

};

// table for custom ascii mapping
const byte asciiMap [128] =
{ 37  ,   //  0 _ NUL
  37  ,   //  1 _ SOH
  37  ,   //  2 _ STX
  37  ,   //  3 _ ETX
  37  ,   //  4 _ EOT
  37  ,   //  5 _ ENQ
  37  ,   //  6 _ ACK
  37  ,   //  7 _ BEL
  37  ,   //  8 _ BS
  37  ,   //  9 _ TAB
  37  ,   //  10  _ LF
  37  ,   //  11  _ VT
  37  ,   //  12  _ FF
  37  ,   //  13  _ CR
  37  ,   //  14  _ SO
  37  ,   //  15  _ SI
  37  ,   //  16  _ DLE
  37  ,   //  17  _ DC1
  37  ,   //  18  _ DC2
  37  ,   //  19  _ DC3
  37  ,   //  20  _ DC4
  37  ,   //  21  _ NAK
  37  ,   //  22  _ SYN
  37  ,   //  23  _ ETB
  37  ,   //  24  _ CAN
  37  ,   //  25  _ EM
  37  ,   //  26  _ SUB
  37  ,   //  27  _ Esc
  37  ,   //  28  _ FS
  37  ,   //  29  _ GS
  37  ,   //  30  _ RS
  37  ,   //  31  _ US
  36  ,   //  32  _ SP
  37  ,   //  33  _ !
  37  ,   //  34  _ "
  37  ,   //  35  _ #
  37  ,   //  36  _ $
  37  ,   //  37  _ %
  37  ,   //  38  _ &
  37  ,   //  39  _ '
  37  ,   //  40  _ (
  37  ,   //  41  _ )
  40  ,   //  42  _ *
  37  ,   //  43  _ +
  39  ,   //  44  _ ,
  37  ,   //  45  _ -
  38  ,   //  46  _ .
  37  ,   //  47  _ /
  0 ,   //  48  _ 0
  1 ,   //  49  _ 1
  2 ,   //  50  _ 2
  3 ,   //  51  _ 3
  4 ,   //  52  _ 4
  5 ,   //  53  _ 5
  6 ,   //  54  _ 6
  7 ,   //  55  _ 7
  8 ,   //  56  _ 8
  9 ,   //  57  _ 9
  37  ,   //  58  _ :
  37  ,   //  59  _ ;
  37  ,   //  60  _ <
  37  ,   //  61  _ =
  37  ,   //  62  _ >
  37  ,   //  63  _ ?
  37  ,   //  64  _ @
  10  ,   //  65  _ A
  11  ,   //  66  _ B
  12  ,   //  67  _ C
  13  ,   //  68  _ D
  14  ,   //  69  _ E
  15  ,   //  70  _ F
  16  ,   //  71  _ G
  17  ,   //  72  _ H
  18  ,   //  73  _ I
  19  ,   //  74  _ J
  20  ,   //  75  _ K
  21  ,   //  76  _ L
  22  ,   //  77  _ M
  23  ,   //  78  _ N
  24  ,   //  79  _ O
  25  ,   //  80  _ P
  26  ,   //  81  _ Q
  27  ,   //  82  _ R
  28  ,   //  83  _ S
  29  ,   //  84  _ T
  30  ,   //  85  _ U
  31  ,   //  86  _ V
  32  ,   //  87  _ W
  33  ,   //  88  _ X
  34  ,   //  89  _ Y
  35  ,   //  90  _ Z
  37  ,   //  91  _ [
  37  ,   //  92  _ backslash
  37  ,   //  93  _ ]
  37  ,   //  94  _ ^
  37  ,   //  95  _ _
  37  ,   //  96  _ `
  10  ,   //  97  _ a
  11  ,   //  98  _ b
  12  ,   //  99  _ c
  13  ,   //  100 _ d
  14  ,   //  101 _ e
  15  ,   //  102 _ f
  16  ,   //  103 _ g
  17  ,   //  104 _ h
  18  ,   //  105 _ i
  19  ,   //  106 _ j
  20  ,   //  107 _ k
  21  ,   //  108 _ l
  22  ,   //  109 _ m
  23  ,   //  110 _ n
  24  ,   //  111 _ o
  25  ,   //  112 _ p
  26  ,   //  113 _ q
  27  ,   //  114 _ r
  28  ,   //  115 _ s
  29  ,   //  116 _ t
  30  ,   //  117 _ u
  31  ,   //  118 _ v
  32  ,   //  119 _ w
  33  ,   //  120 _ x
  34  ,   //  121 _ y
  35  ,   //  122 _ z
  37  ,   //  123 _ {
  37  ,   //  124 _ |
  37  ,   //  125 _ }
  37  ,   //  126 _ ~
  37
} ; //  127 _ DEL

/*
   Character Arrays -
*/

const      byte displayCharactersCCW [CHARNUMBER][CHARWIDTH] =
{
  { 0b00000000  , 0b00000000  , 0b00111110  , 0b01100001  , 0b01010001  , 0b01001001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b01111111  , 0b01100000  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00100001  , 0b01010001  , 0b01010001  , 0b01010001  , 0b01001111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00101110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b01000001  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00010000  , 0b01111111  , 0b00010000  , 0b00010000  , 0b01110000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01001110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b01110001  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01001110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111111  , 0b01000000  , 0b01000000  , 0b01000000  , 0b01000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00101110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b00101110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00111110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b00100001  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00111111  , 0b01010000  , 0b01010000  , 0b01010000  , 0b00111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00101110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000001  , 0b01000001  , 0b01000001  , 0b01000001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00111110  , 0b01000001  , 0b01000001  , 0b01000001  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000001  , 0b01000001  , 0b01010001  , 0b01010001  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01000000  , 0b01000000  , 0b01010000  , 0b01010000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01011110  , 0b01010001  , 0b01000001  , 0b01000001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111111  , 0b00010000  , 0b00010000  , 0b00010000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000001  , 0b01000001  , 0b01111111  , 0b01000001  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111110  , 0b01000001  , 0b01000001  , 0b00000001  , 0b00000110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01000011  , 0b00100100  , 0b00011000  , 0b00010000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000001  , 0b00000001  , 0b00000001  , 0b00000001  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111111  , 0b00100000  , 0b00010000  , 0b00100000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111111  , 0b00001100  , 0b00111000  , 0b01100000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00111110  , 0b01000001  , 0b01000001  , 0b01000001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00100000  , 0b01010000  , 0b01010000  , 0b01010000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00111101  , 0b01000010  , 0b01000001  , 0b01000001  , 0b00111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00100011  , 0b01010100  , 0b01011000  , 0b01010000  , 0b01111111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01001110  , 0b01010001  , 0b01010001  , 0b01010001  , 0b00100001  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01000000  , 0b01000000  , 0b01111111  , 0b01000000  , 0b01000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111110  , 0b00000001  , 0b00000001  , 0b00000001  , 0b01111110  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01110000  , 0b00001110  , 0b00000011  , 0b00001110  , 0b01110000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01111000  , 0b00000111  , 0b00011100  , 0b00000111  , 0b01111000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01000111  , 0b00101000  , 0b00010000  , 0b00101000  , 0b01000111  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01000000  , 0b00100000  , 0b00011111  , 0b00100000  , 0b01000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b01100001  , 0b01010001  , 0b01001001  , 0b01000101  , 0b01000011  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00101000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000001  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00000000  , 0b00000010  , 0b00000001  , 0b00000000  , 0b00000000  , 0b00000000  , 0b00000000  },
  { 0b00000000  , 0b00000000  , 0b00101000  , 0b00010000  , 0b01111100  , 0b00010000  , 0b00101000  , 0b00000000  , 0b00000000  }
};



const      byte displayCharacters [CHARNUMBER][CHARWIDTH] =
{
  { 0b000000000,  0b000111000,  0b001100100,  0b001010100,  0b001001100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b000011000,  0b000011000,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b000010000},
  { 0b000000000,  0b000111100,  0b001000000,  0b000111000,  0b000000100,  0b000000100,  0b000000100,  0b001111100},
  { 0b000000000,  0b000111100,  0b001000000,  0b000111000,  0b001000000,  0b001000000,  0b001000000,  0b000111100},
  { 0b000000000,  0b000100100,  0b000100100,  0b001111100,  0b000100000,  0b000100000,  0b000100000,  0b000100000},
  { 0b000000000,  0b001111100,  0b000000100,  0b000111100,  0b001000000,  0b001000000,  0b001000000,  0b000111100},
  { 0b000000000,  0b001111000,  0b000000100,  0b000111100,  0b001000100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b001111100,  0b001000000,  0b001000000,  0b001000000,  0b001000000,  0b001000000,  0b001000000},
  { 0b000000000,  0b000111000,  0b001000100,  0b000111000,  0b001000100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b000111000,  0b001000100,  0b001111000,  0b001000000,  0b001000000,  0b001000000,  0b000111100},
  { 0b000000000,  0b000111000,  0b001000100,  0b001111100,  0b001000100,  0b001000100,  0b001000100,  0b001000100},
  { 0b000000000,  0b000111100,  0b001000100,  0b000111100,  0b001000100,  0b001000100,  0b001000100,  0b000111100},
  { 0b000000000,  0b000111000,  0b000000100,  0b000000100,  0b000000100,  0b000000100,  0b000000100,  0b001111000},
  { 0b000000000,  0b000111100,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b000111100},
  { 0b000000000,  0b000111100,  0b000000100,  0b000011100,  0b000000100,  0b000000100,  0b000000100,  0b001111100},
  { 0b000000000,  0b001111100,  0b000000100,  0b000011100,  0b000000100,  0b000000100,  0b000000100,  0b000000100},
  { 0b000000000,  0b001111000,  0b000000100,  0b001100100,  0b001000100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b001000100,  0b001000100,  0b001111100,  0b001000100,  0b001000100,  0b001000100,  0b001000100},
  { 0b000000000,  0b000111000,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b001111000},
  { 0b000000000,  0b001110000,  0b001000000,  0b001000000,  0b001000000,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b001000100,  0b000100100,  0b000011100,  0b000010100,  0b000100100,  0b001000100,  0b001000100},
  { 0b000000000,  0b000000100,  0b000000100,  0b000000100,  0b000000100,  0b000000100,  0b000000100,  0b001111100},
  { 0b000000000,  0b001000100,  0b001101100,  0b001010100,  0b001000100,  0b001000100,  0b001000100,  0b001000100},
  { 0b000000000,  0b001001100,  0b001011100,  0b001010100,  0b001110100,  0b001100100,  0b001000100,  0b001000100},
  { 0b000000000,  0b000111000,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b000111100,  0b001000100,  0b000111100,  0b000000100,  0b000000100,  0b000000100,  0b000000100},
  { 0b000000000,  0b000111000,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b000100100,  0b001011000},
  { 0b000000000,  0b000111100,  0b001000100,  0b000111100,  0b000010100,  0b000100100,  0b001000100,  0b001000100},
  { 0b000000000,  0b001111000,  0b000000100,  0b000111000,  0b001000000,  0b001000000,  0b001000000,  0b000111100},
  { 0b000000000,  0b001111100,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b000010000},
  { 0b000000000,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b001000100,  0b000111000},
  { 0b000000000,  0b001000100,  0b001000100,  0b001000100,  0b000101000,  0b000101000,  0b000111000,  0b000010000},
  { 0b000000000,  0b001000100,  0b001000100,  0b001010100,  0b001010100,  0b000111000,  0b000101000,  0b000101000},
  { 0b000000000,  0b001000100,  0b000101000,  0b000010000,  0b000101000,  0b001000100,  0b001000100,  0b001000100},
  { 0b000000000,  0b001000100,  0b000101000,  0b000010000,  0b000010000,  0b000010000,  0b000010000,  0b000010000},
  { 0b000000000,  0b001111100,  0b001000000,  0b000100000,  0b000010000,  0b000001000,  0b000000100,  0b001111100},
  { 0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000},
  { 0b000000000,  0b000000000,  0b000010000,  0b000000000,  0b000010000,  0b000000000,  0b000000000,  0b000000000},
  { 0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000010000},
  { 0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000000000,  0b000100000,  0b000010000},
  { 0b000000000,  0b000010000,  0b001010100,  0b000111000,  0b001010100,  0b000010000,  0b000000000,  0b000000000}
};




char temp[3000];


/*
  ----------------------------------  init --------------------------------------------
*/

void setup(void) {
  Serial.begin(115200);         // Start the Serial communication to send messages to the computer
  delay(10);
  Serial.println('\n');
  pinMode(A0, INPUT);
  brightness = analogRead(A0) / 104;
  randomSeed(analogRead(0));

  /*
     Display inital screen
  */

  strip.begin();

  scrollText("       LED clock ", GREENTONE, STANDARDSCROLLSPEED);
  scrollText(SOURCECODEVERSION, GREENTONE, STANDARDSCROLLSPEED);
  scrollText("      ", GREENTONE, STANDARDSCROLLSPEED);


  wifiMulti.addAP("SSID", "Password");   // add Wi-Fi networks you want to connect to
  

  Serial.print(" Connecting...       ");
  int i = 0;
  while (wifiMulti.run() != WL_CONNECTED) { // Wait for the Wi-Fi to connect: scan for Wi-Fi networks, and connect to the strongest of the networks above
    delay(250);
    scrollText(".", GREENTONE, STANDARDSCROLLSPEED);
  }

  Serial.println('\n');
  Serial.println("Connected to ");
  //scrollText("Connected to ");
  Serial.println(WiFi.SSID());              // Tell us what network we're connected to
  Serial.print("IP address:\t");
  Serial.println(WiFi.localIP());           // Send the IP address of the ESP8266 to the computer

  if (MDNS.begin("esp8266")) {              // Start the mDNS responder for esp8266.local
    Serial.println("mDNS responder started");
  } else {
    Serial.println("Error setting up MDNS responder!");
  }


  ArduinoOTA.onStart([]() {
    String type;
    if (ArduinoOTA.getCommand() == U_FLASH) {
      type = "sketch";
    } else { // U_FS
      type = "filesystem";
    }

    // NOTE: if updating FS this would be the place to unmount FS using FS.end()
    Serial.println("Start updating " + type);
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) {
      Serial.println("Auth Failed");
    } else if (error == OTA_BEGIN_ERROR) {
      Serial.println("Begin Failed");
    } else if (error == OTA_CONNECT_ERROR) {
      Serial.println("Connect Failed");
    } else if (error == OTA_RECEIVE_ERROR) {
      Serial.println("Receive Failed");
    } else if (error == OTA_END_ERROR) {
      Serial.println("End Failed");
    }
  });
  ArduinoOTA.begin();
  Serial.println("Ready");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  server.on("/", HTTP_GET, handleRoot);     // Call the 'handleRoot' function when a client requests URI "/"
  server.on("/showClockVertical", HTTP_POST, showClockVertical);   
  server.on("/kickBall", HTTP_POST, kickBall);   
  server.on("/Previous", HTTP_POST, Previous);  
  server.on("/Next", HTTP_POST, Next);  
  server.on("/resetDevice", HTTP_POST, resetDevice);   
  server.on("/save", HTTP_GET, handleSave);   
  server.onNotFound(handleNotFound);        // When a client requests an unknown URI (i.e. something other than "/"), call function "handleNotFound"

  server.begin();

  // Actually start the server
  Serial.println("HTTP server started");
  Serial.println("Starting UDP");
  Udp.begin(localPort);
  Serial.print("Local port: ");
  Serial.println(Udp.localPort());
  Serial.println("waiting for sync");
  setSyncProvider(getNtpTime);
  setSyncInterval(300);

  scrollText( WiFi.localIP().toString(), GREENTONE, STANDARDSCROLLSPEED);
  scrollText("          ", GREENTONE, STANDARDSCROLLSPEED);
}

/*
   ------------------------- loop ----------------------------------------------------------------
*/


void loop() {

  server.handleClient();
  ArduinoOTA.handle();

  switch (whichChoice) {
    case 1: {
        if (previousChoice != whichChoice) {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Lights Off      ", BLUETONE, SCROLLRIGHT, STANDARDSCROLLSPEED, true);
          ledOff();
        }
        break;
      }
    case 2: {
        if (previousChoice != whichChoice) {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Lights On      ", BLUETONE, SCROLLRIGHT, STANDARDSCROLLSPEED, true);

          for (int i = 0; i < LEDCOUNT; i++) strip.setPixelColor(i, LIGHTTONE);
          strip.show();
        }
        break;
      }
    case 3: {
        if (previousChoice != whichChoice)
        {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Phaser      ");
        }
        for (int i = 0; i < LEDCOUNT; i++) setPixelArray( STANDARDFXLAYER, i, 100 + xPos * 2, 0, 0);
        bounce();
        delay(1);
        break;
      }
    case 4: {
        if (previousChoice != whichChoice) {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Bouncing Ball      ", 0, 0, 128, SCROLLRIGHT, STANDARDSCROLLSPEED, true);
          ledOff();
          xVel = 0.5; yVel = 0.5; xPos = 41; yPos = 1;
        }
        bounce();
        rain(30);


        break;
      }
    case 5: {
        digitalClockDisplay(GREENTONE , SCROLLRIGHT);
        //snow();
        break;
      }
    case 6: {
        digitalClockDisplay(GREENTONE , SCROLLLEFT);
        break;
      }
    case 10: {
        if (previousChoice != whichChoice) {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Police      ", 0, 0, 128, SCROLLRIGHT, STANDARDSCROLLSPEED, true);
        }
        lightOnColor(0, 0, 255);
        delay(50);
        hideScreen();
        delay(100);
        lightOnColor(0, 0, 255);
        delay(50);
        hideScreen();
        delay(100);
        lightOnColor(0, 0, 255);
        delay(50);
        hideScreen();
        delay(500);
        break;
      }

    case 8: {
        if (previousChoice != whichChoice)
        {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Disco      ");
          //Serial.println("Disco");
        }
        byte discoluminosity = random(0, 10);
        lightOnColor(discoluminosity * random(0, 25), discoluminosity * random(0, 25), discoluminosity * random(0, 25));
        int beat = random(1, 4);
        delay(beat * 50);
        ledOff();
        delay(200 - beat);
        break;
      }
    case 9: {
        if (previousChoice != whichChoice)
        {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Snow      ", 128, 128, 128, SCROLLRIGHT, STANDARDSCROLLSPEED, true);
          //Serial.println("Disco");
        }
        snow();
        break;
      }
    case 7: {
        if (previousChoice != whichChoice)
        {
          clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
          scrollText(" Candlelight      ", 128, 60, 0, SCROLLRIGHT, 5, true);
          for (int x = 0; x < LEDCOUNT; x++)
          {

            pixelArray[LAYER3][x][0] = pixelCandle[x][0];
            pixelArray[LAYER3][x][1] = pixelCandle[x][1];
            pixelArray[LAYER3][x][2] = pixelCandle[x][2];
          }
          drawScreen(true);
        }
        candleLight();
      }
    default: {
        if (previousChoice != whichChoice) //Serial.println("Default");
          break;
      }
  }
  previousChoice = whichChoice;

}

/*
   --------------------------------- Functions -------------------------------------------
*/

void candleLight() {
 
  byte xy=random(OUTERFLAMEPIXELS);
  byte rg=random(FLAMEVARIATION)+FLAMEBASE;
  byte divisor=random(1,4);
  setPixelArray(STANDARDFXLAYER, candleOuterFlame[xy][0], candleOuterFlame[xy][1], rg/divisor,(rg-FLAMEREDNESS)/divisor,0);
  xy=random(INNERFLAMEPIXELS);
  setPixelArray(STANDARDFXLAYER, candleInnerFlame[xy][0], candleInnerFlame[xy][1], rg,rg-FLAMEREDNESS,0);
    
  drawScreen(true, false);

}


void snow()
{
  if (snowloop--) return;
  snowloop = SNOWLOOP;

  byte xSnow = random(0, SCREENY - 1);
  for (int x = 0; x < SCREENY; x++)
  {
    pixelArrayBuffer [STANDARDFXLAYER] [x] [0] = (x == xSnow ? 128 : 0);
    pixelArrayBuffer [STANDARDFXLAYER] [x] [1] = (x == xSnow ? 128 : 0);
    pixelArrayBuffer [STANDARDFXLAYER] [x] [2] = (x == xSnow ? 128 : 0);
  }
  scrollScreen(STANDARDFXLAYER, SCROLLLEFT, 0, SCREENX - 1, BUFFERSCROLL);
  if (snowheight++ > SCREENX * 3) // SCREENX*3 number of snowflakes have fallen
  {
    for (int x = 0; x < SCREENY; x++)
    {
      pixelArrayBuffer [LAYER2] [x] [0] = reLight(200);
      pixelArrayBuffer [LAYER2] [x] [1] = reLight(200);
      pixelArrayBuffer [LAYER2] [x] [2] = reLight(200);
    }
    scrollScreen(LAYER2, SCROLLRIGHT, 0, SCREENX - 1, BUFFERSCROLL);
    snowheight = 0;
  }
  drawScreen(true);
}

void rain(byte density)
{
  for (int x = 0; x < SCREENY; x++) {
    pixelArrayBuffer [STANDARDFXLAYER] [x] [0] = 0;
    pixelArrayBuffer [STANDARDFXLAYER] [x] [1] = 0;
    pixelArrayBuffer [STANDARDFXLAYER] [x] [2] = random(0, 255) / density;
  }
  scrollScreen(STANDARDFXLAYER, SCROLLLEFT, 0, SCREENX - 1, BUFFERSCROLL);
}

void bounce()
{
  bounce(GRAVITYON, LAYER2); // Default call with gravity on
}

void bounce(boolean gravity, byte whichlayer)
{
  xPos = xPos + xVel;

  if (xPos > XMAX - 1 ) {
    xPos = XMAX - 1;
    xVel = -xVel;
  }
  if (xPos < 1) {
    xPos = 0;
    xVel = -xVel;
  }
  if (gravity) xVel = xVel + LEDCLOCKGRAVITY;

  if (yPos > YMAX || yPos < 1) yVel = -yVel;
  yPos = yPos + yVel;

  setPixelArray(whichlayer, (int)yPos * SCREENX + (int)xPos , reLight(255), 0, reLight(255));
 

  drawScreen(true);

  delay(STANDARDDELAY);

  setPixelArray(whichlayer, (int)yPos * SCREENX + (int)xPos , 0, 0, 0);

}

void ledOff()
{
  for (int i = 0; i < LEDCOUNT; i++) strip.setPixelColor(i, 0, 0, 0);
  strip.show();

}

/*
    --------------------------------------Draw screen --------------------------------------
    todo:
*/

void drawScreen(boolean showscreen)
{
  drawScreen(showscreen, true);
}

void drawScreen(boolean showscreen, boolean autobright)
{
  //byte brightnessimpact;
  buildPixelArray();
  if (autobright)
  {
 
    brightness = analogRead(A0) / 104;
 
  }
  else
  {
    //brightnessimpact = 1;
    brightness = 9;
  }
  for (int i = 0 ; i < LEDCOUNT; i++)
  {
    strip.setPixelColor(pixel2strip[i], reLight( pixelArrayFinal[i][0]) , reLight( pixelArrayFinal[i][1]) , reLight( pixelArrayFinal[i][2]) );
  }
  if (showscreen)
  {
    strip.show();
#ifdef DEBUG
    for(int x= SCREENX; x>=0 ;x--)
    {
      for( byte y=0; y<SCREENY;y++){
        if(pixelArrayFinal[x+y*SCREENX][0]+pixelArrayFinal[x+y*SCREENX][1]+pixelArrayFinal[x+y*SCREENX][2]) Serial.print("*");
        else Serial.print(" ");
      }
      Serial.println();
    }
#endif

    //delay (1);   maybe reactivate in case of timin issues -> HW resets
  }
}




void buildPixelArray()
{
  for (int i = 0 ; i < LEDCOUNT; i++)
  {
    for (int layer = 0; layer < MAXLAYERS; layer++)
    {
      if (pixelArray[layer][i][0] + pixelArray[layer][i][1] + pixelArray[layer][i][2])
      {
        pixelArrayFinal[i][0] = pixelArray[layer][i][0];
        pixelArrayFinal[i][1] = pixelArray[layer][i][1];
        pixelArrayFinal[i][2] = pixelArray[layer][i][2];
        break;
      }
      else
      {
        pixelArrayFinal[i][0] = 0;
        pixelArrayFinal[i][1] = 0;
        pixelArrayFinal[i][2] = 0;
      }
    }
  }
}

unsigned char lightArray[10] = {250 , 250, 200, 200, 200, 100, 50, 10, 1, 1};

unsigned char reLight(unsigned char color)
{

  unsigned char retval = 0;
  if (color != 0)
  {
    if (color / lightArray[static_cast<unsigned char>(brightness)] != 0) retval = static_cast<unsigned char>(color / lightArray[static_cast<unsigned char>(brightness)]);
    else retval = 1;
  }
 
  return static_cast<unsigned char>(retval);
}

void hideScreen()
{
  lightOnColor(0, 0, 0);
}

void clearScreen(byte whichlayer)
{
  for (int i = 0; i < LEDCOUNT; i++)
  {
    setPixelArray(whichlayer, i, 0, 0, 0);
  }
}



void lightOnColor(byte redtone, byte greentone, byte bluetone)
{
  for (int i = 0 ; i < LEDCOUNT; i++)
    strip.setPixelColor(i, redtone, greentone, bluetone);
  strip.show();
}


void setPixelArray(byte whichlayer, byte xpos, byte ypos, byte redtone, byte greentone, byte bluetone)
{
  setPixelArray(whichlayer, xpos + SCREENX * ypos, redtone, greentone, bluetone);
}

void setPixelArray(byte whichlayer, int yscreenxx, byte redtone, byte greentone, byte bluetone)
{
  pixelArray[whichlayer][yscreenxx][0] = redtone;
  pixelArray[whichlayer][yscreenxx][1] = greentone;
  pixelArray[whichlayer][yscreenxx][2] = bluetone;
}

/* Screen scroll routines


  Todo:


*/

void scrollScreen(byte scrolldirection, byte scrolltype)
{
  scrollScreen(STANDARDTEXTLAYER, scrolldirection, 0, SCREENX - 1, scrolltype);
}

void scrollScreen(byte whichlayer, byte scrolldirection, byte xstartposition, byte xendposition, byte scrolltype) // direction - up,down,left,right; scrolltype - empty, loop or filled by buffer  - fill pixelArrayBuffer array appropriately with pixels and colors to scroll in from if scrolltype is 'filled'
{

  if (xstartposition < 0 || xstartposition > SCREENX - 2 || xendposition < xstartposition || xendposition > SCREENX - 1) return;

  switch (scrolldirection)
  {
    case SCROLLUP: {
        // save existing line to buffer if scrolltype is loop, otherwise buffer will contain prepared values
        if (scrolltype == LOOPSCROLL)
        {
          //          for (int x = 0; x < SCREENX; x++)
          for (int x = xstartposition; x <= xendposition; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][x][color] = pixelArray[whichlayer][x][color];
          }
        }
        for (int y = 1; y < SCREENY; y++)
        {
          //          for (int x = 0; x < SCREENX; x++)
          for (int x = xstartposition; x <= xendposition; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArray[whichlayer][x + SCREENX * (y - 1)][color] = pixelArray[whichlayer][x + SCREENX * y][color];
          }
        }
        if (scrolltype == EMPTYSCROLL)
        {
          for (int x = 0; x < SCREENX; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][x][color] = 0;
          }
        }
        //        for (int x = 0; x < SCREENX; x++)
        for (int x = xstartposition; x <= xendposition; x++)
        {
          for (int color = 0; color <= 2; color++) pixelArray[whichlayer][x + SCREENX * (SCREENY - 1)][color] = pixelArrayBuffer[whichlayer][x][color];
        }
        break;
      }
    case SCROLLDOWN: {
        // save existing line to buffer if scrolltype is loop, otherwise buffer will contain prepared values
        if (scrolltype == LOOPSCROLL)
        {
          for (int x = 0; x < SCREENX; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][x][color] = pixelArray[whichlayer][x + SCREENX * (SCREENY - 1)][color];
          }
        }
        for (int y = SCREENY - 1; y > 0 ; y--)
        {
          for (int x = 0; x < SCREENX; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArray[whichlayer][x + SCREENX * (y)][color] = pixelArray[whichlayer][x + SCREENX * (y - 1)][color];
          }
        }
        if (scrolltype == EMPTYSCROLL)
        {
          for (int x = 0; x < SCREENX; x++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][x ][color] = 0;
          }
        }
        for (int x = 0; x < SCREENX; x++)
        {
          for (int color = 0; color <= 2; color++) pixelArray[whichlayer][x][color] = pixelArrayBuffer[whichlayer][x][color];
        }
        break;
      }
    case SCROLLLEFT: {
        if (scrolltype == LOOPSCROLL)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][y][color] = pixelArray[whichlayer][SCREENX * y][color];
          }
        }
        for (int x = 1; x < SCREENX ; x++)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArray[whichlayer][(x - 1) + SCREENX * y][color] = pixelArray[whichlayer][(x) + SCREENX * y][color];
          }
        }
        if (scrolltype == EMPTYSCROLL)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][y][color] = 0;
          }
        }
        for (int y = 0; y < SCREENY; y++)
        {
          for (int color = 0; color <= 2; color++) pixelArray[whichlayer][(SCREENX - 1) + SCREENX * y][color] = pixelArrayBuffer[whichlayer][y][color];
        }
        break;
      }
    case SCROLLRIGHT: {
        if (scrolltype == LOOPSCROLL)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][y][color] = pixelArray[whichlayer][(SCREENX - 1) + SCREENX * y][color];
          }
        }
        for (int x = SCREENX - 1; x > 0 ; x--)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArray[whichlayer][(x) + SCREENX * y][color] = pixelArray[whichlayer][(x - 1) + SCREENX * y][color];
          }
        }
        if (scrolltype == EMPTYSCROLL)
        {
          for (int y = 0; y < SCREENY; y++)
          {
            for (int color = 0; color <= 2; color++) pixelArrayBuffer[whichlayer][y ][color] = 0;
          }
        }
        for (int y = 0; y < SCREENY; y++)
        {
          for (int color = 0; color <= 2; color++) pixelArray[whichlayer][SCREENX * y][color] = pixelArrayBuffer[whichlayer][y][color];
        }
        break;
      }
    default: {};
  }
}



/*
   Feed Character into PixelBuffer
   Provide scrolldirection to enable calling scroll routine
   todo: gerealize for direction scroll, currently only SCROLLLEFT
*/


void pixelPrintChar(byte charascii, byte redtone, byte greentone, byte bluetone, byte scrolldirection, byte scrollspeed, boolean drawscreen )
{
  pixelPrintChar(STANDARDTEXTLAYER, charascii, redtone, greentone, bluetone, scrolldirection, scrollspeed , drawscreen);
}


void pixelPrintChar(byte whichlayer, byte charascii, byte redtone, byte greentone, byte bluetone, byte scrolldirection, byte scrollspeed , boolean drawscreen)
{
  /*
     Set parameters for scroll direction
     use const      byte displayCharactersCCW [CHARNUMBER][CHARWIDTH]
  */

  byte charheight = CHARHEIGHT;
  byte charwidth = CHARWIDTH - 1;
  byte charinterleave = 1; //how many pixels to group
  byte displaychar = charascii; // keep charascii unchanged for further processing purpose
  byte charmask = 0b10000000; // enable
  byte charmaskbuffer = charmask;
  byte offset = 1;
  byte verticaloffset = 0;
  boolean rotatecharacters = true;

  switch (scrolldirection)
  {
    case SCROLLUP: {
        charheight = CHARHEIGHT;
        charwidth = 7;
        charinterleave = 5; //how many pixels to group
        charmask = 0b00000001;
        rotatecharacters = false;
        break;
      }
    case SCROLLDOWN: {
        charheight = CHARHEIGHT;
        charwidth = 7;
        charinterleave = 5; //how many pixels to group
        charmask = 0b00000001;
        rotatecharacters = false;
        break;
      }
    case SCROLLLEFT: {
        charheight = CHARHEIGHT;
        charwidth = CHARWIDTH - 1;
        charinterleave = 1; //how many pixels to group
        charmask = 0b00000001;
        rotatecharacters = true;
        break;
      }
    case SCROLLRIGHT: {
        charheight = CHARHEIGHT;
        charwidth = CHARWIDTH;
        charinterleave = 1; //how many pixels to group
        charmask = 0b10000000;
        rotatecharacters = false;
        offset = 1;
        verticaloffset = 0;
        break;
      }
    default: {};
  }

  for (byte horizontal = offset; horizontal < charwidth; horizontal ++)
  {
    charmaskbuffer = charmask;
    for (byte vertical = verticaloffset; vertical < charheight ; vertical++)
    {
      //      if (displayCharactersCCW [charascii][charwidth - 1 - horizontal] & charmaskbuffer)
      if (rotatecharacters == true ?  (boolean) {
      displayCharactersCCW [charascii][charwidth - 1 - horizontal] & charmaskbuffer
      } : (boolean) {
        displayCharacters [charascii][horizontal] & charmaskbuffer
      })
      {
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][0] = redtone;
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][1] = greentone;
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][2] = bluetone;
        //      Serial.print("*");
      }
      else
      {
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][0] = 0;
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][1] = 0;
        pixelArrayBuffer[whichlayer][charheight - 1 - vertical][2] = 0;
        //       Serial.print(" ");
      }
      (rotatecharacters ? charmaskbuffer = charmaskbuffer << 1 : charmaskbuffer = charmaskbuffer >>1 );
    }
    scrollScreen(scrolldirection, BUFFERSCROLL);
    //   Serial.println();
    if (drawscreen) {
      drawScreen(true);
      delay (scrollspeed);
    }
  }

}


/*
    --------------------------------------Print text--------------------
    todo:
*/

void scrollText(String text2print)
{
  scrollText(text2print, BLUETONE, STANDARDSCROLLSPEED);
}

void scrollText(String text2print, byte redtone, byte greentone, byte bluetone, byte scrollspeed)
{
  scrollText(text2print, redtone, greentone, bluetone, SCROLLLEFT, scrollspeed, true);
}

void scrollText(String text2print, byte redtone, byte greentone, byte bluetone, byte scrolldirection, byte scrollspeed, boolean drawtext)
{
  for (int stringposition = 0; stringposition < text2print.length() ; stringposition++)
  {
    pixelPrintChar(asciiMap[text2print.charAt(stringposition)],  redtone, greentone, bluetone, scrolldirection, scrollspeed, drawtext);
  }
}



void digitalClockDisplay(byte redtone, byte greentone, byte bluetone)
{
  digitalClockDisplay(redtone, greentone, bluetone, SCROLLLEFT);
}

void digitalClockDisplay(byte redtone, byte greentone, byte bluetone, byte scrolldirection)
{
  String timestring = " ";
  byte xstartposition = 0;
  byte xendposition = SCREENX - 1;

  if (lastsecond == second())
  {
     return;
  }

  lastsecond = second();

  timestring = timestring + hour() / 10;
  timestring = timestring + hour() % 10;
  timestring = timestring + ":";
  timestring = timestring + minute() / 10;
  timestring = timestring + minute() % 10;



  scrollText(timestring, redtone, bluetone, greentone, scrolldirection, FASTSCROLL, false);
  if (scrolldirection == SCROLLLEFT) // scroll two pixels to center time display
  {
    scrollScreen(scrolldirection, EMPTYSCROLL);
    scrollScreen(scrolldirection, EMPTYSCROLL);
  }
  drawScreen(true);
}

void flipCharacter(byte startposition, byte endposition)
{
  flipCharacter(STANDARDTEXTLAYER, startposition, endposition);
}

void flipCharacter(byte whichscreen, byte startposition, byte endposition)
{
  for (byte i = 0 ; i < SCREENY; i++)
  {
    scrollScreen(whichscreen, SCROLLUP, startposition, endposition, LOOPSCROLL);
    drawScreen(true);
    delay(STANDARDDELAY);
  }
}

void digitalClockDisplay()
{
  digitalClockDisplay(128, 128, 128, SCROLLRIGHT);
}


/*
    --------------------------------------Web Server functions--------------------
    todo:
*/


void handleRoot()
{ // When URI / is requested, send a web page with a button to toggle the LED

  snprintf(temp, 3000, "<body>\<h1>LED light strip Controller</h1>  \   
                       </body> </html> \
                       <p style=\"font-size:60px\"> \
                       <form action=\"/showClockVertical\" method=\"POST\"><input type=\"submit\" value=\"Clock vertical\" style=\"width:350px; font-size:50px\" ></form>  \
                       <form action=\"/Previous\" method=\"POST\"><input type=\"submit\" value=\"Previous\" style=\"width:350px; font-size:50px\" ></form> \
                       <form action=\"/Next\" method=\"POST\"><input type=\"submit\" value=\"Next\" style=\"width:350px; font-size:50px\" ></form> \
                       <form action=\"/resetDevice\" method=\"POST\"><input type=\"submit\" value=\"Reset\" style=\"width:350px; font-size:50px\" ></form> \
                       <form action=\"/kickBall\" method=\"POST\"><input type=\"submit\" value=\"Kicker\" style=\"width:350px; font-size:50px\" ></form>  \
                        <input type='text' name='date_hh' id='date_hh' size=16 autofocus  style=\"width:350px; font-size:50px\" >  \
                        <div>\
                        <br><button style=\"width:350px; font-size:50px\" id=\"save_button\">Submit</button>\
                        </div>\
                           <script src=\"https://ajax.googleapis.com/ajax/libs/jquery/1.11.3/jquery.min.js\"></script>\
                            <script>\
                              var hh;\
                              $('#save_button').click(function(e){\
                                e.preventDefault();\
                                hh = $('#date_hh').val();\
                                $.get('/save?hh=' + hh , function(data){\
                                  console.log(data);\
                                });\
                              });\
                            </script>\                        
                         </body>\
                        " , hour(), minute() % 60, second() % 60);
  server.send(3000, "text/html", temp);
}

void handleSave()
{
  scrollText("      ");
  scrollText(server.arg("hh"));
  scrollText("      ");
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP sstrip.show();tatus 303 (See Other) to redirect
}

void Next() {                          // If a POST request is made to URI /LED
  if (whichChoice < MAXCHOICE) whichChoice++ ; else whichChoice = 1;
  //  Serial.println(whichChoice);
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP sstrip.show();tatus 303 (See Other) to redirect
}
void Previous ()
{ // If a POST request is made to URI /LED
  if (whichChoice > 1) whichChoice-- ; else whichChoice = MAXCHOICE;
  //  Serial.println(whichChoice);
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP status 303 (See Other) to redirect
}

void resetDevice()
{
  setSyncProvider(getNtpTime);
  if (whichChoice == 5 || whichChoice == 6)
  {
    clearScreen(LAYER2);
    clearScreen(LAYER3);
  }
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP status 303 (See Other) to redirect

}

void kickBall()
{
  xVel = xVel + 4.3;
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP status 303 (See Other) to redirect

}

void showClockVertical()
{
  clearScreen(LAYER1); clearScreen(LAYER2); clearScreen(LAYER3);
  whichChoice=5;
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP status 303 (See Other) to redirect

}

void printHTMLText()
{
  server.sendHeader("Location", "/");       // Add a header to respond with a new location for the browser to go to the home page again
  server.send(303);                         // Send it back to the browser with an HTTP status 303 (See Other) to redirect

}

/*
    --------------------------------------------------------
    todo:
*/

void handleNotFound() {
  server.send(404, "text/plain", "404: Not found"); // Send HTTP status 404 (Not Found) when there's no handler for the URI in the request
}

/*
    ----------------------------------------------------------
    todo:
*/




/*
    ---------------------------NTP code-----------------------
    todo:
*/

/*-------- NTP code ----------*/

const int NTP_PACKET_SIZE = 48; // NTP time is in the first 48 bytes of message
byte packetBuffer[NTP_PACKET_SIZE]; //buffer to hold incoming & outgoing packets

time_t getNtpTime()
{
  IPAddress ntpServerIP; // NTP server's ip address

  while (Udp.parsePacket() > 0) ; // discard any previously received packets
  Serial.println("Transmit NTP Request");
  // get a random server from the pool
  WiFi.hostByName(ntpServerName, ntpServerIP);
  Serial.print(ntpServerName);
  Serial.print(": ");
  Serial.println(ntpServerIP);
  sendNTPpacket(ntpServerIP);
  uint32_t beginWait = millis();
  while (millis() - beginWait < 1500) {
    int size = Udp.parsePacket();
    if (size >= NTP_PACKET_SIZE) {
      Serial.println("Receive NTP Response");
      Udp.read(packetBuffer, NTP_PACKET_SIZE);  // read packet into the buffer
      unsigned long secsSince1900;
      // convert four bytes starting at location 40 to a long integer
      secsSince1900 =  (unsigned long)packetBuffer[40] << 24;
      secsSince1900 |= (unsigned long)packetBuffer[41] << 16;
      secsSince1900 |= (unsigned long)packetBuffer[42] << 8;
      secsSince1900 |= (unsigned long)packetBuffer[43];
      return secsSince1900 - 2208988800UL + timeZone * SECS_PER_HOUR;
    }
  }
  Serial.println("No NTP Response :-(");
  return 0; // return 0 if unable to get the time
}


/*
    ----------------------------------------------------------
    todo:
*/

// send an NTP request to the time server at the given address
void sendNTPpacket(IPAddress & address)
{
  // set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);
  // Initialize values needed to form NTP request
  // (see URL above for details on the packets)
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;
  // all NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  Udp.beginPacket(address, 123); //NTP requests are to port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}
